----------------------------------------------------------------------------------
-- Company: 		comp. engr. KKU.
-- Engineer: 		TK
-- 
-- Create Date:    09:55:02 08/22/2011 
-- Design Name: 
-- Module Name:    WordALU_tb - Behavioral 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--   "If you think you can do a thing or think you can't do a thing, you're right."
--   - Henry Ford 

----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity WordALU_tb is
end WordALU_tb;

architecture Behavioral of WordALU_tb is

component WordALU is
    Port ( a, b : in  STD_LOGIC_VECTOR (31 downto 0);
           Binvert, CarryIn : in  STD_LOGIC;
           Operation : in  STD_LOGIC_VECTOR (1 downto 0);
           Result : out  STD_LOGIC_VECTOR (31 downto 0);
           Overflow, Zero : out  STD_LOGIC);
end component;

	signal a, b : STD_LOGIC_VECTOR (31 downto 0);
	signal Binvert, CarryIn : STD_LOGIC;
	signal Operation : STD_LOGIC_VECTOR (1 downto 0);
	signal Result : STD_LOGIC_VECTOR (31 downto 0);
	signal Overflow, Zero : STD_LOGIC;

   constant PERIOD : time := 100 ns;

begin
	
	walu: WordALU
	port map (
		a => a, 
		b => b,
		Binvert => Binvert, 
		CarryIn => CarryIn,
		Operation => Operation,
		Result => Result,
		Overflow => Overflow, 
		Zero => Zero);

	PROCESS
   BEGIN

		Binvert <= '0';
		CarryIn <= '0';


	-- AND --
		Operation <= "00";	-- 00: AND, 01: OR, 10: ADDITION, 11: Less (slt)
		a 			<= "00111111111100000110000001101010";
		b 			<= "01011011110000001010010011110010";
		-- expect:  "00011011110000000010000001100010" = '0x1bc02062'
		WAIT FOR PERIOD;
			
	-- OR --
		Operation <= "01";	-- 00: AND, 01: OR, 10: ADDITION, 11: Less (slt)
		a 			<= "00111111111100000110000001101010";
		b 			<= "01011011110000001010010011110010";
		-- expect:  "01111111111100001110010011111010" = '0x7ff0e4fa'
		WAIT FOR PERIOD;
	
	-- ADDITION --
		-- A + B; A > 0, B > 0, no overflow
		
		Binvert <= '0';		-- A + B
		CarryIn <= '0';
		Operation <= "10";	-- 00: AND, 01: OR, 10: ADDITION, 11: Less (slt)
		
		a 			<= "00111111111100000110000001101010";
		b 			<= "00111111110000001010010011110010";
		-- expect:	"01111111101100010000010101011100" = 0x7fb1055c
		
		WAIT FOR PERIOD;

		-- A + B; A > 0, B > 0, overflow

		Binvert <= '0';		-- A + B
		CarryIn <= '0';
		Operation <= "10";	-- 00: AND, 01: OR, 10: ADDITION, 11: Less (slt)
		
		a 			<= "01111111111100000110000001101010";
		b 			<= "00111111110000001010010011110010";
		-- expect:	"10111111101100010000010101011100" = 0xbfb1055c
		
		WAIT FOR PERIOD;
		
		-- A + B; A < 0, B < 0, no overflow

		Binvert <= '0';		-- A + B
		CarryIn <= '0';
		Operation <= "10";	-- 00: AND, 01: OR, 10: ADDITION, 11: Less (slt)
		
		a 			<= "11000100011001010011011000000000"; -- = 0xc4653600 = -1000000000 
		b 			<= "10111011101000100010101101000000"; -- = 0xbba22b40 = -1147000000
		-- expect: "d10000000000001110110000101000000"; -- = 0x80076140 = -2147000000
		
		WAIT FOR PERIOD;
		
		-- A + B; A > 0, B < 0

		Binvert <= '0';		-- A + B
		CarryIn <= '0';
		Operation <= "10";	-- 00: AND, 01: OR, 10: ADDITION, 11: Less (slt)
		
		a 			<= "01000100011001010011011000000000"; -- = 0x44653600 =  1147483648 
		b 			<= "10111011101000100010101101000000"; -- = 0xbba22b40 = -1147000000
		-- expect: "?00000000000001110110000101000000"; -- = 0x00076140 = 483648
		
		WAIT FOR PERIOD;
		
		-- A + B; A < 0, B > 0

		Binvert <= '0';		-- A + B
		CarryIn <= '0';
		Operation <= "10";	-- 00: AND, 01: OR, 10: ADDITION, 11: Less (slt)
		
		a 			<= "10111011101000100010101101000000"; -- = 0xbba22b40 = -1147000000
		b 			<= "01000100011001010011011000000000"; -- = 0x44653600 =  1147483648 
		-- expect: "?00000000000001110110000101000000"; -- = 0x00076140 = 483648

		WAIT FOR PERIOD;
		
		-- A - B; A > 0, B > 0
		
		Binvert <= '1';		-- A - B
		CarryIn <= '1';
		Operation <= "10";	-- 00: AND, 01: OR, 10: ADDITION, 11: Less (slt)
		
		a 			<= "00111111111100000110000001101010"; -- = 0x3ff0606a 	= 1072717930
		b 			<= "00111111110000001010010011110010"; -- = 0x3fc0a4f2 	= 1069589746
		-- expect:	"00000000001011111011101101111000" 		= 0x002fbb78	=    3128184

		WAIT FOR PERIOD;

		-- A - B; A > 0, B < 0, no overflow

		Binvert <= '1';		-- A - B
		CarryIn <= '1';
		Operation <= "10";	-- 00: AND, 01: OR, 10: ADDITION, 11: Less (slt)
		
		a 			<= "00111111111100000110000001101010"; -- = 0x3ff0606a 	= 1072717930
		b 			<= "11111111111111101100011110000000"; -- = 0xfffec780 	=     -80000
		-- expect:	"00111111111100011001100011101010" 		= 0x3ff198ea	= 1072797930

		WAIT FOR PERIOD;
		
		-- A - B; A > 0, B < 0, overflow

		Binvert <= '1';		-- A - B
		CarryIn <= '1';
		Operation <= "10";	-- 00: AND, 01: OR, 10: ADDITION, 11: Less (slt)		

		a 			<= "01111111111100000110000001101010"; -- = 0x7ff0606a	= 2146459754
		b 			<= "11111111110000101111011100000000"; -- = 0xffc2f700	=   -4000000
		-- expect:  "10000000001011010110100101101010";		= 0x802d696a	; 2150459754

		WAIT FOR PERIOD;
		
		-- A - B; A < 0, B < 0
		
			-- <input code for this test case>
		
		-- A - B; A < 0, B > 0, no overflow
		
			-- <input code for this test case>
		
		-- A - B; A < 0, B > 0, overflow

		Binvert <= '1';		-- A - B
		CarryIn <= '1';
		Operation <= "10";	-- 00: AND, 01: OR, 10: ADDITION, 11: Less (slt)		

		a 			<= "11111111110000101111011100000000"; -- = 0xffc2f700	=    -4000000
		b 			<= "01111111111100000110000001101010"; -- = 0x7ff0606a	= 	2146459754
		-- expect: "?01111111110100101001011010010110";		= 0x7fd29696	; -2150459754

		WAIT FOR PERIOD;

	-- LESS (slt) --

		Operation <= "11";	-- 00: AND, 01: OR, 10: ADDITION, 11: Less (slt)		
		
		-- a < b
		a	<= "00000000000000000000000000000101"; -- 5
		b	<= "00000000000000000000000000011100"; -- 28
		WAIT FOR PERIOD;

		-- a = b
		a	<= "00000000000000000000000000110110"; -- 54
		b	<= "00000000000000000000000000110110"; -- 54
		WAIT FOR PERIOD;
		
		-- a > b
		a	<= "00000000000000000000000000110110"; -- 54
		b	<= "00000000000000000000000000011100"; -- 28
		WAIT FOR PERIOD;
		
		-- a < b, a < 0, b < 0
		a 	<= "11111111111111111111110111011111"; -- -545
		b	<= "11111111111111111111111011010100"; -- -300
		WAIT FOR PERIOD;
		
		-- a < b, a < 0, b > 0
		a 	<= "11111111111111111111110111011111"; -- -545
		b  <= "00000000000000000000000100101100"; -- 300
		WAIT FOR PERIOD;
		
		-- simulation time: 1600 + 100 ns
		
   END PROCESS;


end Behavioral;

