----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    19:18:21 09/04/2011 
-- Design Name: 
-- Module Name:    ch5mem - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 	 to provide interface of 32-bit address/data
--						 It allows only addresses: 
--						 1000 0000 to 1000 00FF for the data segment
--						 and 0040 0000 to 0040 00FF for the program segment (full range: 0040 0000-0FFF FFFF)
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
--use IEEE.STD_LOGIC_ARITH.ALL;

entity ch5mem is
	port
	(
		clock				: in  std_logic;
		datain			: in  std_logic_vector(31 downto 0);
		memaddress		: in  std_logic_vector(31 downto 0);
		memwrite			: in  std_logic;
		memread			: in  std_logic;
		dataout			: out std_logic_vector(31 downto 0);
		enable			: in std_logic;
		startup			: in  std_logic	-- to load pre-written data
	);

end ch5mem;

architecture Behavioral of ch5mem is
	type RAMROM is array(0 TO 2 ** 8 - 1) of std_logic_vector(31 downto 0);
	signal data_block : RAMROM;
	signal program_block: RAMROM;
	
begin

	process(enable, clock, memread, memwrite, memaddress, startup)

	variable index: integer range 0 to 7;
	begin

		if startup = '1' then
			-- initialize the ROM contents
			program_block(0) <= x"8e080000"; -- [00400000]					lw $t0, 0($s0)		
			program_block(1) <= x"8e090004"; -- [00400004]					lw $t1, 4($s0)	
			program_block(2) <= x"0109502a"; -- [00400008]					slt $t2, $t0, $t1
			program_block(3) <= x"11400003"; -- [0040000C]					beq $t2, $zero, LAB_t1ELTt0
			program_block(4) <= x"01285822"; -- [00400010]					sub $t3, $t1, $t0
			program_block(5) <= x"08100007"; -- [00400014]					j LAB_SAVE
			program_block(6) <= x"01095822"; -- [00400018] LAB_t1ELTt0:	sub $t3, $t0, $t1
			program_block(7) <= x"ae0b0008"; -- [0040001C] LAB_SAVE:		sw $t3, 8($s0)
			program_block(8) <= x"8e080008"; -- [00400020] 					lw $t0, 8($s0) # to check if it is well-saved in memory
			
			-- initialize the data content 
			-- This may not sound practical, but since we don't have any instruction dealing with a constant yet.
			-- So, it is for simplicity sake of homework.
			data_block(0) <= x"00000019";		-- [10000000]
			data_block(1) <= x"00000037";		-- [10000004]
														-- [10000008] 
														-- after run, it should has a value, abs( data_block(0) - data_block(1) ) = x"0000001E"
			
		end if;

		-- 0x10000xXX [0001 0000 0000 0000 0000 00xx xxxx xx--]=> data segment; 
		-- 0x00400xXX [0000 0000 0100 0000 0000 00xx xxxx xx--]=> program segment

		index := to_integer(unsigned( memaddress(9 downto 2) ));

		if (enable = '1' and memread = '1') then

			if memaddress(31 downto 10) 		= "0001000000000000000000" then
				dataout <= data_block(index);
			elsif memaddress(31 downto 10) 	= "0000000001000000000000" then
				dataout <= program_block(index);
			else
				dataout <= (dataout'range => 'Z');
			end if;
			   		   
		elsif (enable = '1' and memwrite = '1' and clock'event and clock = '0') then

			if memaddress(31 downto 10) 	= "0001000000000000000000" then
				data_block(index) <= datain;
			elsif memaddress(31 downto 10) = "0000000001000000000000" then
				program_block(index) <= datain;
			end if;

			dataout <= (dataout'range => 'Z');

		else
			dataout <= (dataout'range => 'Z');
		end if;
		
	end process;

end Behavioral;

