----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    17:22:42 09/05/2011 
-- Design Name: 
-- Module Name:    ch5path_tb - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity ch5path_tb is
end ch5path_tb;

architecture Behavioral of ch5path_tb is

-- output of datapath --
signal		clock				: std_logic;
signal		memdatain		: std_logic_vector(31 downto 0);
signal		memdataout		: std_logic_vector(31 downto 0);
signal		memaddress		: std_logic_vector(31 downto 0);
signal		memwrite			: std_logic;
signal		memread			: std_logic;
signal		startup			: std_logic;							-- to load a start-up setting

------------------------

-- test bench component --

	component ch5mem is
	port
	(
		clock				: in  std_logic;
		datain			: in  std_logic_vector(31 downto 0);
		memaddress		: in  std_logic_vector(31 downto 0);
		memwrite			: in  std_logic;
		memread			: in  std_logic;
		dataout			: out std_logic_vector(31 downto 0);
		enable			: in std_logic;
		startup			: in  std_logic	-- to load pre-written data
	);
	end component;

-- original components --


component WordALU is
    Port ( a, b : in  std_logic_vector (31 downto 0);
           Binvert, CarryIn : in  std_logic;
           Operation : in  std_logic_vector (1 downto 0);
           Result : out  std_logic_vector (31 downto 0);
           Overflow, Zero : out  std_logic);
end component;

component mux2 is
port(
	sel: 		in std_logic;
	ch1:		in std_logic_vector(31 downto 0);
	ch2:		in std_logic_vector(31 downto 0);
	muxout:	out std_logic_vector (31 downto 0)
);
end component;

component mux4 is
port(
	sel: 		in std_logic_vector(1 downto 0);
	ch1:		in std_logic_vector(31 downto 0);
	ch2:		in std_logic_vector(31 downto 0);
	ch3:		in std_logic_vector(31 downto 0);
	ch4:		in std_logic_vector(31 downto 0);	
	muxout:	out std_logic_vector (31 downto 0)
);
end component;

component nbitregister is
	 generic (n : integer := 32);
    Port ( 
		win: 			in std_logic_vector (31 downto 0);
		clock:		in std_logic;
		regload:		in std_logic;
		regclear: 	in std_logic;
		wout:			out std_logic_vector (31 downto 0)
);
end component;

component regfile is
port(
	readreg1:		in std_logic_vector (4 downto 0); 	-- index referring to a specific register
	readreg2:		in std_logic_vector (4 downto 0); 	-- index referring to a specific register
	writereg:		in std_logic_vector (4 downto 0); 	-- index referring to a specific register
	writedata:		in std_logic_vector (31 downto 0);	-- data to put in the specified register's content
	readdata1:		out std_logic_vector (31 downto 0);	-- data contained in register indexed by readreg1
	readdata2:		out std_logic_vector (31 downto 0);	-- data contained in register indexed by readreg2
	RegWrite:		in std_logic;
	clock:			in std_logic
);
end component;

component signextend
    port(
         halfword : in  std_logic_vector(15 downto 0);
         fullword : out  std_logic_vector(31 downto 0)
        );
end component;

COMPONENT maincontrol
PORT(
      PCWriteCond : OUT  std_logic;
      PCWrite 		: OUT  std_logic;
      PCSource 	: OUT  std_logic_vector(1 downto 0);
      MemRead 		: OUT  std_logic;
      MemWrite 	: OUT  std_logic;
      IorD 			: OUT  std_logic;
      IRWrite 		: OUT  std_logic;
      ALUop 		: OUT  std_logic_vector(1 downto 0);
      ALUSrcB 		: OUT  std_logic_vector(1 downto 0);
      ALUSrcA 		: OUT  std_logic;
      RegWrite 	: OUT  std_logic;
      RegDst 		: OUT  std_logic;
      MemtoReg 	: OUT  std_logic;
      stagein 		: IN  std_logic_vector(2 downto 0);
      opcode 		: IN  std_logic_vector(5 downto 0);
		stageout 	: OUT  std_logic_vector(2 downto 0)
	);
END COMPONENT;


component ALUControl
    port(
         ALUOp 		: in  std_logic_vector(1 downto 0);
         funct 		: in  std_logic_vector(5 downto 0);
         Binvert 		: out  std_logic;
         Operation 	: out  std_logic_vector(1 downto 0)
        );
end component;

----------------------------------------------------------------------
-- signals --
----------------------------------------------------------------------

signal clk	: std_logic;

-- register file --
signal readreg1	: std_logic_vector (4 downto 0);
signal readreg2	: std_logic_vector (4 downto 0);
signal writereg	: std_logic_vector (4 downto 0); 	-- we use only 4 bits, but this would allow it for being used with a 32-bit mux.
signal writedata	: std_logic_vector (31 downto 0);
signal readdata1	: std_logic_vector (31 downto 0);
signal readdata2	: std_logic_vector (31 downto 0);

-- alu --
signal a, b 				: std_logic_vector (31 downto 0);
signal Binvert, CarryIn : std_logic;
signal Operation 			: std_logic_vector (1 downto 0);
signal Result 				: std_logic_vector (31 downto 0);
signal Overflow, Zero 	: std_logic;

-- registers --
signal PCin				: std_logic_vector (31 downto 0);
signal PCload			: std_logic;
signal PCout, IRout, MDRout, Aout, Bout, ALUoutput		: std_logic_vector (31 downto 0);

-- main control signals --
signal RegWrite	: std_logic;
signal IRWrite		: std_logic;
signal ALUSrcB		: std_logic_vector (1 downto 0);
signal PCSource	: std_logic_vector (1 downto 0);
signal IorD, RegDst, MemtoReg, ALUSrcA : std_logic;
signal ALUOp		: std_logic_vector (1 downto 0);
signal PCWrite, PCWriteCond	: std_logic;

-- pathway signals --

signal SignEx		: std_logic_vector (31 downto 0);

signal adapterMX0		: std_logic_vector (31 downto 0);
signal adapterMX1		: std_logic_vector (31 downto 0);
signal adapterMXout	: std_logic_vector (31 downto 0);

signal jaddr		: std_logic_vector (31 downto 0);
signal beqoffset	: std_logic_vector (31 downto 0);

signal PCSourceOut	: std_logic_vector (31 downto 0);

signal stagein32, stageout32 : std_logic_vector (31 downto 0);

-- simulation variables --

   -- Clock period definitions
   constant clock_period : time := 100 ns;
 

begin

	clk <= clock;

	regs: regfile
	port map (
		readreg1		=> readreg1,
		readreg2		=> readreg2,
		writereg		=> writereg,
		writedata	=> writedata,
		readdata1 	=> readdata1,
		readdata2	=> readdata2,
		RegWrite 	=> RegWrite,
		clock			=> clk);

	alu: WordALU
	port map (
		a => a, 
		b => b,
		Binvert => Binvert, 
		CarryIn => CarryIn,
		Operation => Operation,
		Result => Result,
		Overflow => Overflow, 
		Zero => Zero);

	PC: nbitregister
	port map (
		win => PCin, 
		clock => clk,
		regload => PCload, 
		regclear => '0',
		wout => PCout);

	IR: nbitregister
	port map (
		win => memdatain, 
		clock => clk,
		regload => IRWrite, 
		regclear => startup,
		wout => IRout);

	MDR: nbitregister
	port map (
		win => memdatain, 
		clock => clk,
		regload => '1', 
		regclear => startup,
		wout => MDRout);

	Areg: nbitregister
	port map (
		win => readdata1, 
		clock => clk,
		regload => '1', 
		regclear => startup,
		wout => Aout);

	Breg: nbitregister
	port map (
		win => readdata2, 
		clock => clk,
		regload => '1', 
		regclear => startup,
		wout => Bout);

	ALUout: nbitregister
	port map (
		win => Result, 
		clock => clk,
		regload => '1', 
		regclear => startup,
		wout => ALUoutput);

	mxsrcB: mux4
	port map (
		sel		=> ALUSrcB,
		ch1		=> Bout,								-- R-type
		ch2		=> x"00000004",					-- PC increment
		ch3		=> SignEx,							-- lw/sw
		ch4		=> beqoffset,						-- branch address
		muxout	=> b);

	beqoffset <= SignEx(29 downto 0) & "00";

	mxsrcPC: mux4
	port map (
		sel		=> PCSource,
		ch1		=> Result,														-- PC <= PC + 4
		ch2		=> ALUoutput,													-- beq
		ch3		=> jaddr,														-- j
		ch4		=> x"00400000",												-- boot up address
		muxout	=> PCSourceOut);

	jaddr <= PCOut(31 downto 28) & IRout(25 downto 0) & "00";

	mxIorD: mux2
	port map (
		sel		=> IorD,
		ch1		=> PCout,			-- instruction access
		ch2		=> ALUOutput,		-- memory access (lw/sw)
		muxout	=> memaddress);
		
	mxRegDst: mux2
	port map (
		sel		=> RegDst,
		ch1		=> adapterMX0,	-- lw
		ch2		=> adapterMX1, -- R-type
		muxout	=> adapterMXout);
		
		adapterMX0 <= "000000000000000000000000000" & IRout(20 downto 16);
		adapterMX1 <= "000000000000000000000000000" & IRout(15 downto 11);
		writereg <= adapterMXout(4 downto 0);
		
		
	mxMemtoReg: mux2
	port map (
		sel		=> MemtoReg,
		ch1		=> ALUOutput,	-- R-type
		ch2		=> MDRout,		-- lw
		muxout	=> writedata);

	mxsrcA: mux2
	port map (
		sel		=> ALUSrcA,
		ch1		=> PCout,	-- PC increment / branch address
		ch2		=> Aout,		-- R-type, lw, sw
		muxout	=> a);

	PCin <= PCSourceOut;

	SE: signextend
   port map (
		halfword => IRout(15 downto 0),
      fullword => SignEx);

   alucon: ALUControl 
	port map (
          ALUOp => ALUOp,
          funct => IRout(5 downto 0),
          Binvert => Binvert,
          Operation => Operation);

	CarryIn <= Binvert;	-- Both Binvert and CarryIn indicate subtraction (of sub, slt, and comparing for beq) when set. 
									-- Both are reset for other operations.

   maincon: maincontrol 
	port map (
          PCWriteCond => PCWriteCond,
          PCWrite => PCWrite,
          PCSource => PCSource,
          MemRead => MemRead,
          MemWrite => MemWrite,
          IorD => IorD,
          IRWrite => IRWrite,
          ALUop => ALUop,
          ALUSrcB => ALUSrcB,
          ALUSrcA => ALUSrcA,
          RegWrite => RegWrite,
          RegDst => RegDst,
          MemtoReg => MemtoReg,
          stagein => stagein32(2 downto 0),
          opcode => IRout(31 downto 26),
          stageout => stageout32(2 downto 0));

	stageout32(31 downto 3) <= "00000000000000000000000000000";
	
	statereg: nbitregister
	port map (
		win => stageout32, 
		clock => clk,
		regload => '1', 
		regclear => startup,
		wout => stagein32);


	-- nodes --
	memdataout <= Bout;
	
	readreg1 <= IRout(25 downto 21);
	readreg2 <= IRout(20 downto 16);
	
	PCload <= PCWrite or (PCWriteCond and Zero);
		
-- Simulation part --

 	mem: ch5mem
	port map (
		clock 		=> clock, 
		datain 		=> memdataout,
		memaddress 	=> memaddress, 
		memwrite 	=> memwrite,
		memread		=> memread,
		dataout 		=> memdatain,
		enable		=> '1',
		startup		=> startup);

   -- Clock process definitions
   clock_process :process
   begin
		clock <= '0';
		wait for clock_period/2;
		clock <= '1';
		wait for clock_period/2;
   end process;
 

   -- Stimulus process
   stim_proc: process
   begin		
	
		startup <= '0';
		wait for clock_period/10;

		startup <= '1';
      wait for clock_period/10;	
		
		startup <= '0';
		wait;
   end process;

	-- simulation time = 4000 ns
end Behavioral;