----------------------------------------------------------------------------------
-- Company: 		comp. engr. KKU.
-- Engineer: 		TK
-- 
-- Create Date:    14:27:05 09/03/2011 
-- Design Name: 
-- Module Name:    datapath - Behavioral 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
-- "If your actions inspire others to dream more, learn more, do more and become more, you are a leader."
-- - John Quincy Adams
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity datapath is

	port
	(
		clock				: in  std_logic;
		memdatain		: in  std_logic_vector(31 downto 0);
		memdataout		: out std_logic_vector(31 downto 0);
		memaddress		: out  std_logic_vector(31 downto 0);
		memwrite			: out  std_logic;
		memread			: out  std_logic;
		startup			: in  std_logic;							-- to load a start-up setting
		stagecheck		: out std_logic_vector(2 downto 0);	-- to allow convenient stage monitoring
		PCcheck			: out  std_logic_vector(31 downto 0);
		IRcheck			: out  std_logic_vector(31 downto 0);
		Acheck			: out  std_logic_vector(31 downto 0);
		Bcheck			: out  std_logic_vector(31 downto 0);
		ALUOutcheck		: out  std_logic_vector(31 downto 0);
		MDRcheck			: out  std_logic_vector(31 downto 0)
	);


end datapath;

architecture Behavioral of datapath is

component WordALU is
    Port ( a, b : in  std_logic_vector (31 downto 0);
           Binvert, CarryIn : in  std_logic;
           Operation : in  std_logic_vector (1 downto 0);
           Result : out  std_logic_vector (31 downto 0);
           Overflow, Zero : out  std_logic);
end component;

component mux2 is
port(
	sel: 		in std_logic;
	ch1:		in std_logic_vector(31 downto 0);
	ch2:		in std_logic_vector(31 downto 0);
	muxout:	out std_logic_vector (31 downto 0)
);
end component;

component mux4 is
port(
	sel: 		in std_logic_vector(1 downto 0);
	ch1:		in std_logic_vector(31 downto 0);
	ch2:		in std_logic_vector(31 downto 0);
	ch3:		in std_logic_vector(31 downto 0);
	ch4:		in std_logic_vector(31 downto 0);	
	muxout:	out std_logic_vector (31 downto 0)
);
end component;

component nbitregister is
	 generic (n : integer := 32);
    Port ( 
		win: 			in std_logic_vector (31 downto 0);
		clock:		in std_logic;
		regload:		in std_logic;
		regclear: 	in std_logic;
		wout:			out std_logic_vector (31 downto 0)
);
end component;

component regfile is
port(
	readreg1:		in std_logic_vector (4 downto 0); 	-- index referring to a specific register
	readreg2:		in std_logic_vector (4 downto 0); 	-- index referring to a specific register
	writereg:		in std_logic_vector (4 downto 0); 	-- index referring to a specific register
	writedata:		in std_logic_vector (31 downto 0);	-- data to put in the specified register's content
	readdata1:		out std_logic_vector (31 downto 0);	-- data contained in register indexed by readreg1
	readdata2:		out std_logic_vector (31 downto 0);	-- data contained in register indexed by readreg2
	RegWrite:		in std_logic;
	clock:			in std_logic
);
end component;

component signextend
    port(
         halfword : in  std_logic_vector(15 downto 0);
         fullword : out  std_logic_vector(31 downto 0)
        );
end component;


signal clk	: std_logic;

-- register file --
signal readreg1	: std_logic_vector (4 downto 0);
signal readreg2	: std_logic_vector (4 downto 0);
signal writereg	: std_logic_vector (4 downto 0); 	-- we use only 4 bits, but this would allow it for being used with a 32-bit mux.
signal writedata	: std_logic_vector (31 downto 0);
signal readdata1	: std_logic_vector (31 downto 0);
signal readdata2	: std_logic_vector (31 downto 0);

-- alu --
signal a, b 				: std_logic_vector (31 downto 0);
signal Binvert, CarryIn : std_logic;
signal Operation 			: std_logic_vector (1 downto 0);
signal Result 				: std_logic_vector (31 downto 0);
signal Overflow, Zero 	: std_logic;

-- registers --
signal PCin				: std_logic_vector (31 downto 0);
signal PCload			: std_logic;
signal PCout, IRout, MDRout, Aout, Bout, ALUoutput		: std_logic_vector (31 downto 0);

-- main control signals --
signal RegWrite	: std_logic;
signal IRWrite		: std_logic;
signal ALUSrcB		: std_logic_vector (1 downto 0);
signal PCSource	: std_logic_vector (1 downto 0);
signal IorD, RegDst, MemtoReg, ALUSrcA : std_logic;
signal ALUOp		: std_logic_vector (1 downto 0);
signal PCWrite, PCWriteCond	: std_logic;
signal StartPC		: std_logic; -- to reset PC to the boot-up address

-- pathway signals --

signal SignEx		: std_logic_vector (31 downto 0);

signal adapterMX0		: std_logic_vector (31 downto 0);
signal adapterMX1		: std_logic_vector (31 downto 0);
signal adapterMXout	: std_logic_vector (31 downto 0);

signal jaddr		: std_logic_vector (31 downto 0);
signal beqoffset	: std_logic_vector (31 downto 0);

signal PCSourceOut	: std_logic_vector (31 downto 0);

signal stage : std_logic_vector (2 downto 0);

begin

	clk <= clock;

	regs: regfile
	port map (
		readreg1		=> readreg1,
		readreg2		=> readreg2,
		writereg		=> writereg,
		writedata	=> writedata,
		readdata1 	=> readdata1,
		readdata2	=> readdata2,
		RegWrite 	=> RegWrite,
		clock			=> clk);

	alu: WordALU
	port map (
		a => a, 
		b => b,
		Binvert => Binvert, 
		CarryIn => CarryIn,
		Operation => Operation,
		Result => Result,
		Overflow => Overflow, 
		Zero => Zero);

	PC: nbitregister
	port map (
		win => PCin, 
		clock => clk,
		regload => PCload, 
		regclear => '0',
		wout => PCout);

	IR: nbitregister
	port map (
		win => memdatain, 
		clock => clk,
		regload => IRWrite, 
		regclear => startup,
		wout => IRout);

	MDR: nbitregister
	port map (
		win => memdatain, 
		clock => clk,
		regload => '1', 
		regclear => startup,
		wout => MDRout);

	Areg: nbitregister
	port map (
		win => readdata1, 
		clock => clk,
		regload => '1', 
		regclear => startup,
		wout => Aout);

	Breg: nbitregister
	port map (
		win => readdata2, 
		clock => clk,
		regload => '1', 
		regclear => startup,
		wout => Bout);

	ALUout: nbitregister
	port map (
		win => Result, 
		clock => clk,
		regload => '1', 
		regclear => startup,
		wout => ALUoutput);

	mxsrcB: mux4
	port map (
		sel		=> ALUSrcB,
		ch1		=> Bout,								-- R-type
		ch2		=> x"00000004",					-- PC increment
		ch3		=> SignEx,							-- lw/sw
		ch4		=> beqoffset,						-- branch address
		muxout	=> b);

	beqoffset <= SignEx(29 downto 0) & "00";

	mxsrcPC: mux4
	port map (
		sel		=> PCSource,
		ch1		=> Result,														-- PC <= PC + 4
		ch2		=> ALUoutput,													-- beq
		ch3		=> jaddr,														-- j
		ch4		=> x"80000180",												-- exception
		muxout	=> PCSourceOut);

	jaddr <= PCOut(31 downto 28) & IRout(25 downto 0) & "00";

	mxIorD: mux2
	port map (
		sel		=> IorD,
		ch1		=> PCout,			-- instruction access
		ch2		=> ALUOutput,		-- memory access (lw/sw)
		muxout	=> memaddress);
		
	mxRegDst: mux2
	port map (
		sel		=> RegDst,
		ch1		=> adapterMX0,	-- lw
		ch2		=> adapterMX1, -- R-type
		muxout	=> adapterMXout);
		
		adapterMX0 <= "000000000000000000000000000" & IRout(20 downto 16);
		adapterMX1 <= "000000000000000000000000000" & IRout(15 downto 11);
		writereg <= adapterMXout(4 downto 0);
		
		
	mxMemtoReg: mux2
	port map (
		sel		=> MemtoReg,
		ch1		=> ALUOutput,	-- R-type
		ch2		=> MDRout,		-- lw
		muxout	=> writedata);

	mxsrcA: mux2
	port map (
		sel		=> ALUSrcA,
		ch1		=> PCout,	-- PC increment / branch address
		ch2		=> Aout,		-- R-type, lw, sw
		muxout	=> a);

	mxStartPC: mux2
	port map (
		sel		=> StartPC,
		ch1		=> PCSourceOut,	-- updated address
		ch2		=> x"00400000",	-- boot-up address
		muxout	=> PCin);


	SE: signextend
   port map (
		halfword => IRout(15 downto 0),
      fullword => SignEx);


	-- nodes --
	memdataout <= Bout;
	
	readreg1 <= IRout(25 downto 21);
	readreg2 <= IRout(20 downto 16);
	
	PCload <= PCWrite or (PCWriteCond and Zero) or startup;
	
	StartPC <= startup;
	
	ALUControl: process(IRout(5 downto 0), ALUOp)
	begin
	
		case ALUop is
		when "00" =>
			-- lw/sw: add --
			Binvert <= '0';
			Operation <= "10";
		when "01" =>
			-- beq: sub --
			Binvert <= '1';
			Operation <= "10";
		when "10" =>
			-- R-type --
			case IRout(3 downto 0) is
			when "0000" =>
			-- add --
			Binvert <= '0';
			Operation <= "10";
			
			when "0010" =>
			-- sub --
			Binvert <= '1';
			Operation <= "10";
			
			when "0100" =>
			-- and --
			Binvert <= '0';
			Operation <= "00";			
			
			when "0101" =>
			-- or --
			Binvert <= '0';
			Operation <= "01";
			
			when "1010" =>
			-- slt --
			Binvert <= '1';
			Operation <= "11";
						
			when others => Binvert <= '0'; Operation <= "00"; 
			end case;
		when others => 
			-- default case --
			Binvert <= '0';
			Operation <= "00";
		end case;
		
		CarryIn <= Binvert;	-- Both Binvert and CarryIn indicate subtraction (of sub, slt, and comparing for beq) when set. 
									-- Both are reset for other operations.
		
		
	end process;

	MainControl: process
	
	begin
	
	-- normal operation --

		case stage is
		when "001" =>
			-- Instruction Fetch --
			PCWriteCond <= '0';	PCWrite <= '1';	IorD <= '0';
			MemRead <= '1';		MemWrite <= '0';	MemtoReg <= '0';
			IRWrite <= '1';		PCSource <= "00";	
			ALUop <= "00";			ALUSrcB <= "01";		ALUSrcA <= '0';	
			RegWrite <= '0';		RegDst <= '0';
			
			stage <= "010";
		when "010" =>
			-- Instruction Decode & Brach Address Calculation --
			PCWriteCond <= '0';	PCWrite <= '0';	IorD <= '0';
			MemRead <= '0';		MemWrite <= '0';	MemtoReg <= '0';
			IRWrite <= '0';		PCSource <= "00";	
			ALUop <= "00";			ALUSrcB <= "11";		ALUSrcA <= '0';	
			RegWrite <= '0';		RegDst <= '0';
		
			stage <= "011";
		when "011" =>

			case IRout(31 downto 26) is
			when "000000" =>
				-- r-type --
				PCWriteCond <= '0';	PCWrite <= '0';	IorD <= '0';
				MemRead <= '0';		MemWrite <= '0';	MemtoReg <= '0';
				IRWrite <= '0';		PCSource <= "00";	
				ALUop <= "10";			ALUSrcB <= "00";		ALUSrcA <= '1';	
				RegWrite <= '0';		RegDst <= '0';	

				stage <= "100";
			
			when "100011" =>
				-- lw --
				PCWriteCond <= '0';	PCWrite <= '0';	IorD <= '0';
				MemRead <= '0';		MemWrite <= '0';	MemtoReg <= '0';
				IRWrite <= '0';		PCSource <= "00";	
				ALUop <= "00";			ALUSrcB <= "10";		ALUSrcA <= '1';	
				RegWrite <= '0';		RegDst <= '0';				
				
				stage <= "100";
		
			when "101011" =>
				-- sw --
				PCWriteCond <= '0';	PCWrite <= '0';	IorD <= '0';
				MemRead <= '0';		MemWrite <= '0';	MemtoReg <= '0';
				IRWrite <= '0';		PCSource <= "00";	
				ALUop <= "00";			ALUSrcB <= "10";		ALUSrcA <= '1';	
				RegWrite <= '0';		RegDst <= '0';				
			
				stage <= "100";
			
			when "000100" =>
				-- beq --
				PCWriteCond <= '1';	PCWrite <= '0';	IorD <= '0';
				MemRead <= '0';		MemWrite <= '0';	MemtoReg <= '0';
				IRWrite <= '0';		PCSource <= "01";	
				ALUop <= "01";			ALUSrcB <= "00";		ALUSrcA <= '1';	
				RegWrite <= '0';		RegDst <= '0';		

				stage <= "001";
			
			when "000010" =>
				-- j --
				PCWriteCond <= '0';	PCWrite <= '1';	IorD <= '0';
				MemRead <= '0';		MemWrite <= '0';	MemtoReg <= '0';
				IRWrite <= '0';		PCSource <= "10";	
				ALUop <= "00";			ALUSrcB <= "00";		ALUSrcA <= '0';	
				RegWrite <= '0';		RegDst <= '0';				

				stage <= "001";

			when others =>
				-- unknown operation --
				stage <= "001";
			end case;
		when "100" =>
			case IRout(31 downto 26) is
			when "000000" =>
				-- r-type --
				PCWriteCond <= '0';	PCWrite <= '0';	IorD <= '0';
				MemRead <= '0';		MemWrite <= '0';	MemtoReg <= '0';
				IRWrite <= '0';		PCSource <= "00";	
				ALUop <= "00";			ALUSrcB <= "00";		ALUSrcA <= '0';	
				RegWrite <= '1';		RegDst <= '1';	

				stage <= "001";
			
			when "100011" =>
				-- lw --
				PCWriteCond <= '0';	PCWrite <= '0';	IorD <= '1';
				MemRead <= '1';		MemWrite <= '0';	MemtoReg <= '0';
				IRWrite <= '0';		PCSource <= "00";	
				ALUop <= "00";			ALUSrcB <= "00";		ALUSrcA <= '0';	
				RegWrite <= '0';		RegDst <= '0';				
				
				stage <= "101";
		
			when "101011" =>
				-- sw --
				PCWriteCond <= '0';	PCWrite <= '0';	IorD <= '1';
				MemRead <= '0';		MemWrite <= '1';	MemtoReg <= '0';
				IRWrite <= '0';		PCSource <= "00";	
				ALUop <= "00";			ALUSrcB <= "00";		ALUSrcA <= '0';	
				RegWrite <= '0';		RegDst <= '0';				
			
				stage <= "001";
			
			when others =>
				-- unknown operation --
				stage <= "001";
			end case;
		when "101" =>
		
			if IRout(31 downto 26) = "100011" then
				-- lw --
				PCWriteCond <= '0';	PCWrite <= '0';	IorD <= '0';
				MemRead <= '0';		MemWrite <= '0';	MemtoReg <= '1';
				IRWrite <= '0';		PCSource <= "00";	
				ALUop <= "00";			ALUSrcB <= "00";		ALUSrcA <= '0';	
				RegWrite <= '1';		RegDst <= '0';				
				
			end if;
		
			stage <= "001";
		when others =>
			-- default --
			stage <= "001";
		end case;
		
		wait on clk until clk = '1';
		
	end process;

	stagecheck <= stage;

	PCcheck <= PCout;
	IRcheck <= IRout;
	Acheck <= Aout;
	Bcheck <= Bout;
	ALUOutcheck <= ALUOutput;
	MDRcheck <= MDRout;

end Behavioral;

