----------------------------------------------------------------------------------
-- Company: 		comp. engr. KKU.
-- Engineer: 		TK
-- 
-- Create Date:    07:41:53 09/06/2011 
-- Design Name: 
-- Module Name:    maincontrol - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--	"Truth will set you free." - James A. Garfield
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity maincontrol is

	port(
		PCWriteCond : out  std_logic;
		PCWrite 		: out  std_logic;
		PCSource 	: out std_logic_vector(1 downto 0);
		MemRead 		: out  std_logic;
		MemWrite 	: out  std_logic;
		IorD 			: out  std_logic;
		IRWrite 		: out  std_logic;
		ALUop 		: out std_logic_vector(1 downto 0);
		ALUSrcB 		: out std_logic_vector(1 downto 0);
		ALUSrcA 		: out  std_logic;
		RegWrite 	: out  std_logic;
		RegDst 		: out  std_logic;
		MemtoReg 	: out  std_logic;
		stagein 		: in  std_logic_vector(2 downto 0);
		opcode		: in 	std_logic_vector(5 downto 0); -- IRout(31 downto 26)
		stageout 	: out std_logic_vector(2 downto 0)	);

end maincontrol;

architecture Behavioral of maincontrol is

begin

	MainControl: process(stagein)
	
	begin
	
		case stagein is
		when "000" =>
			-- boot up --
			PCWrite <= '1';	PCSource <= "11";
			MemRead <= '0';	MemWrite <= '0';	IRWrite <= '0';	RegWrite <= '0';
			
			stageout <= "001";

		when "001" =>
			-- Instruction Fetch --
			PCWriteCond <= '0';	PCWrite <= '1';	IorD <= '0';
			MemRead <= '1';		MemWrite <= '0';	MemtoReg <= '0';
			IRWrite <= '1';		PCSource <= "00";	
			ALUop <= "00";			ALUSrcB <= "01";		ALUSrcA <= '0';	
			RegWrite <= '0';		RegDst <= '0';
			
			stageout <= "010";
		when "010" =>
			-- Instruction Decode & Brach Address Calculation --
			PCWriteCond <= '0';	PCWrite <= '0';	IorD <= '0';
			MemRead <= '0';		MemWrite <= '0';	MemtoReg <= '0';
			IRWrite <= '0';		PCSource <= "00";	
			ALUop <= "00";			ALUSrcB <= "11";		ALUSrcA <= '0';	
			RegWrite <= '0';		RegDst <= '0';
		
			stageout <= "011";
		when "011" =>

			case opcode is
			when "000000" =>
				-- r-type --
				PCWriteCond <= '0';	PCWrite <= '0';	IorD <= '0';
				MemRead <= '0';		MemWrite <= '0';	MemtoReg <= '0';
				IRWrite <= '0';		PCSource <= "00";	
				ALUop <= "10";			ALUSrcB <= "00";		ALUSrcA <= '1';	
				RegWrite <= '0';		RegDst <= '0';	

				stageout <= "100";
			
			when "100011" =>
				-- lw --
				PCWriteCond <= '0';	PCWrite <= '0';	IorD <= '0';
				MemRead <= '0';		MemWrite <= '0';	MemtoReg <= '0';
				IRWrite <= '0';		PCSource <= "00";	
				ALUop <= "00";			ALUSrcB <= "10";		ALUSrcA <= '1';	
				RegWrite <= '0';		RegDst <= '0';				
				
				stageout <= "100";
		
			when "101011" =>
				-- sw --
				PCWriteCond <= '0';	PCWrite <= '0';	IorD <= '0';
				MemRead <= '0';		MemWrite <= '0';	MemtoReg <= '0';
				IRWrite <= '0';		PCSource <= "00";	
				ALUop <= "00";			ALUSrcB <= "10";		ALUSrcA <= '1';	
				RegWrite <= '0';		RegDst <= '0';				
			
				stageout <= "100";
			
			when "000100" =>
				-- beq --
				PCWriteCond <= '1';	PCWrite <= '0';	IorD <= '0';
				MemRead <= '0';		MemWrite <= '0';	MemtoReg <= '0';
				IRWrite <= '0';		PCSource <= "01";	
				ALUop <= "01";			ALUSrcB <= "00";		ALUSrcA <= '1';	
				RegWrite <= '0';		RegDst <= '0';		

				stageout <= "001";
			
			when "000010" =>
				-- j --
				PCWriteCond <= '0';	PCWrite <= '1';	IorD <= '0';
				MemRead <= '0';		MemWrite <= '0';	MemtoReg <= '0';
				IRWrite <= '0';		PCSource <= "10";	
				ALUop <= "00";			ALUSrcB <= "00";		ALUSrcA <= '0';	
				RegWrite <= '0';		RegDst <= '0';				

				stageout <= "001";

			when others =>
				-- unknown operation --
				stageout <= "000";
			end case;
		when "100" =>
			case opcode is
			when "000000" =>
				-- r-type --
				PCWriteCond <= '0';	PCWrite <= '0';	IorD <= '0';
				MemRead <= '0';		MemWrite <= '0';	MemtoReg <= '0';
				IRWrite <= '0';		PCSource <= "00";	
				ALUop <= "00";			ALUSrcB <= "00";		ALUSrcA <= '0';	
				RegWrite <= '1';		RegDst <= '1';	

				stageout <= "001";
			
			when "100011" =>
				-- lw --
				PCWriteCond <= '0';	PCWrite <= '0';	IorD <= '1';
				MemRead <= '1';		MemWrite <= '0';	MemtoReg <= '0';
				IRWrite <= '0';		PCSource <= "00";	
				ALUop <= "00";			ALUSrcB <= "00";		ALUSrcA <= '0';	
				RegWrite <= '0';		RegDst <= '0';				
				
				stageout <= "101";
		
			when "101011" =>
				-- sw --
				PCWriteCond <= '0';	PCWrite <= '0';	IorD <= '1';
				MemRead <= '0';		MemWrite <= '1';	MemtoReg <= '0';
				IRWrite <= '0';		PCSource <= "00";	
				ALUop <= "00";			ALUSrcB <= "00";		ALUSrcA <= '0';	
				RegWrite <= '0';		RegDst <= '0';				
			
				stageout <= "001";
			
			when others =>
				-- unknown operation --
				stageout <= "000";
			end case;
		when "101" =>
		
			if opcode = "100011" then
				-- lw --
				PCWriteCond <= '0';	PCWrite <= '0';	IorD <= '0';
				MemRead <= '0';		MemWrite <= '0';	MemtoReg <= '1';
				IRWrite <= '0';		PCSource <= "00";	
				ALUop <= "00";			ALUSrcB <= "00";		ALUSrcA <= '0';	
				RegWrite <= '1';		RegDst <= '0';				
				
			end if;
		
			stageout <= "001";
		when others =>
			-- default --
			stageout <= "000";
		end case;
		
	end process;

end Behavioral;

