----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    17:06:36 09/03/2011 
-- Design Name: 
-- Module Name:    mempack - Behavioral 
-- Project Name: 
-- Description: 	 to provide interface of 32-bit address/data
--						 This is a small version of a memory pack.
--						 It allows only addresses: 
--						 1000 0000 to 1000 FFFF for the data segment
--						 and 0040 0000 to 0040 FFFF for the program segment (full range: 0040 0000-0FFF FFFF)
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity mempack is
	generic
	(
		loadprogram		: integer := 1	-- '1': load a predefined program into program memory.
	);
	port
	(
		clock				: IN  std_logic;
		datain32			: IN  std_logic_vector(31 DOWNTO 0);
		memaddress32	: IN  std_logic_vector(31 DOWNTO 0);
		memwrite			: IN  std_logic;
		memread			: IN  std_logic;
		dataout32		: OUT std_logic_vector(31 DOWNTO 0)		
	);
end mempack;

architecture Behavioral of mempack is

component memunit is
	generic
	(
		ADDRESS_WIDTH	: integer := 16;	-- somehow, simulation tool can not handle any value larger than 16 bits.
		DATA_WIDTH		: integer := 8
	);
	port
	(
		clock				: IN  std_logic;
		datain			: IN  std_logic_vector(DATA_WIDTH - 1 DOWNTO 0);
		memaddress		: IN  std_logic_vector(ADDRESS_WIDTH - 1 DOWNTO 0);
		memwrite			: IN  std_logic;
		memread			: IN  std_logic;
		dataout			: OUT std_logic_vector(DATA_WIDTH - 1 DOWNTO 0);
		enable			: IN std_logic
	);
end component;

component mux2 is
port(
	sel: 		in std_logic;
	ch1:		in std_logic_vector(31 downto 0);
	ch2:		in std_logic_vector(31 downto 0);
	muxout:	out std_logic_vector (31 downto 0)
);
end component;

signal	clk			: std_logic;
signal 	dataEN		: std_logic;
signal 	programEN	: std_logic;

signal	Addr32		: std_logic_vector(31 DOWNTO 0);

signal	datainB0		: std_logic_vector(7 DOWNTO 0);
signal	datainB1		: std_logic_vector(7 DOWNTO 0);
signal	datainB2		: std_logic_vector(7 DOWNTO 0);
signal	datainB3		: std_logic_vector(7 DOWNTO 0);

signal	QdataB0		: std_logic_vector(7 DOWNTO 0);
signal	QdataB1		: std_logic_vector(7 DOWNTO 0);
signal	QdataB2		: std_logic_vector(7 DOWNTO 0);
signal	QdataB3		: std_logic_vector(7 DOWNTO 0);

signal	QprogramB0		: std_logic_vector(7 DOWNTO 0);
signal	QprogramB1		: std_logic_vector(7 DOWNTO 0);
signal	QprogramB2		: std_logic_vector(7 DOWNTO 0);
signal	QprogramB3		: std_logic_vector(7 DOWNTO 0);

signal	Qdata32			: std_logic_vector(31 DOWNTO 0);
signal	Qprogram32		: std_logic_vector(31 DOWNTO 0);
signal	Qmux				: std_logic_vector(31 DOWNTO 0);

begin

	clk <= clock;
	datainB3 <= datain32(31 downto 24);
	datainB2 <= datain32(23 downto 16);
	datainB1 <= datain32(15 downto 8);
	datainB0 <= datain32(7 downto 0);
	
	Addr32	<=	memaddress32;
		
	mux: mux2
	port map (
	-- 0x10000000 [0001 0000 0000 0000 0000 0000 0000 0000]=> data segment; 
	-- 0x00400000 [0000 0000 0100 0000 0000 0000 0000 0000]=> program segment
		sel		=> Addr32(28),
		ch1		=> Qprogram32, 
		ch2		=> Qdata32, 
		muxout	=> Qmux);
	
	Qprogram32 	<= QprogramB3 	& QprogramB2 	& QprogramB1 	& QprogramB0;
	Qdata32 		<= QdataB3 		& QdataB2 		& QdataB1 		& QdataB0;
	dataout32 	<= Qmux;
	
	dataEN		<= Addr32(28);
	programEN	<= not Addr32(28);
		
	dmem3: memunit
	port map (
		clock 		=> clk, 
		datain 		=> datainB3,
		memaddress 	=> Addr32(17 downto 2), -- byte @ "11" of 4 of 2**16 x 8 byte memories;
		memwrite 	=> memwrite,
		memread		=> memread,
		dataout 		=> QdataB3,
		enable		=> dataEN);
		
	dmem2: memunit
	port map (
		clock 		=> clk, 
		datain 		=> datainB2,
		memaddress 	=> Addr32(17 downto 2), -- byte @ "10" of 4 of 2**16 x 8 byte memories;
		memwrite 	=> memwrite,
		memread		=> memread,
		dataout 		=> QdataB2,
		enable		=> dataEN);

	dmem1: memunit
	port map (
		clock 		=> clk, 
		datain 		=> datainB1,
		memaddress 	=> Addr32(17 downto 2), -- byte @ "01" of 4 of 2**16 x 8 byte memories;
		memwrite 	=> memwrite,
		memread		=> memread,
		dataout 		=> QdataB1,
		enable		=> dataEN);

	dmem0: memunit
	port map (
		clock 		=> clk, 
		datain 		=> datainB0,
		memaddress 	=> Addr32(17 downto 2), -- byte @ "00" of 4 of 2**16 x 8 byte memories;
		memwrite 	=> memwrite,
		memread		=> memread,
		dataout 		=> QdataB0,
		enable		=> dataEN);

	prog3: memunit
	port map (
		clock 		=> clk, 
		datain 		=> datainB3,
		memaddress 	=> Addr32(17 downto 2), -- byte @ "11" of 4 of 2**16 x 8 byte memories;
		memwrite 	=> memwrite,
		memread		=> memread,
		dataout 		=> QprogramB3,
		enable		=> programEN);
		
	prog2: memunit
	port map (
		clock 		=> clk, 
		datain 		=> datainB2,
		memaddress 	=> Addr32(17 downto 2), -- byte @ "10" of 4 of 2**16 x 8 byte memories;
		memwrite 	=> memwrite,
		memread		=> memread,
		dataout 		=> QprogramB2,
		enable		=> programEN);
		
	prog1: memunit
	port map (
		clock 		=> clk, 
		datain 		=> datainB1,
		memaddress 	=> Addr32(17 downto 2), -- byte @ "01" of 4 of 2**16 x 8 byte memories;
		memwrite 	=> memwrite,
		memread		=> memread,
		dataout 		=> QprogramB1,
		enable		=> programEN);

	prog0: memunit
	port map (
		clock 		=> clk, 
		datain 		=> datainB0,
		memaddress 	=> Addr32(17 downto 2), -- byte @ "00" of 4 of 2**16 x 8 byte memories;
		memwrite 	=> memwrite,
		memread		=> memread,
		dataout 		=> QprogramB0,
		enable		=> programEN);

end Behavioral;

