-- adapted from http://www.altera.com/support/examples/vhdl/vhd-single-clock-syncram.html
-- retrieved on Aug 29th, 2011.

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;

ENTITY memunit IS
	GENERIC
	(
		ADDRESS_WIDTH	: integer := 32;
		DATA_WIDTH		: integer := 8
	);
	PORT
	(
		clock				: IN  std_logic;
		datain			: IN  std_logic_vector(DATA_WIDTH - 1 DOWNTO 0);
		memaddress		: IN  std_logic_vector(ADDRESS_WIDTH - 1 DOWNTO 0);
		memwrite			: IN  std_logic;
		memread			: IN  std_logic;
		dataout			: OUT std_logic_vector(DATA_WIDTH - 1 DOWNTO 0);
		enable			: IN std_logic
	);
END memunit;

ARCHITECTURE rtl OF memunit IS
	TYPE RAM IS ARRAY(0 TO 2 ** ADDRESS_WIDTH - 1) OF std_logic_vector(DATA_WIDTH - 1 DOWNTO 0);

	SIGNAL ram_block : RAM;
BEGIN
	PROCESS (clock, enable)
	BEGIN
			IF (enable = '1' and memwrite = '1') THEN
			   
				ram_block(to_integer(unsigned(memaddress))) <= datain;
				dataout <= (dataout'range => 'Z');
		   
			ELSIF (enable = '1' and memread = '1') THEN

				dataout <= ram_block(to_integer(unsigned(memaddress)));			
			
			ELSE
				dataout <= (dataout'range => 'Z');
			END IF;
	END PROCESS;
END rtl;
