----------------------------------------------------------------------------------
-- Company: 		comp. engr. KKU.
-- Engineer: 		TK
-- 
-- Create Date:    08/31/2011 
-- Design Name: 
-- Module Name:    memunit_tb - Behavioral 
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity memunit_tb is
end memunit_tb;

architecture Behavioral of memunit_tb is

component memunit is
	generic
	(
		ADDRESS_WIDTH	: integer := 16;		-- simulation overflow when using address_width 32
		DATA_WIDTH		: integer := 8
	);
	port
	(
		clock				: IN  std_logic;
		datain			: IN  std_logic_vector(DATA_WIDTH - 1 DOWNTO 0);
		memaddress		: IN  std_logic_vector(ADDRESS_WIDTH - 1 DOWNTO 0);
		memwrite			: IN  std_logic;
		memread			: IN  std_logic;
		dataout			: OUT std_logic_vector(DATA_WIDTH - 1 DOWNTO 0);
		enable			: IN  std_logic
	);
end component;

signal	clk			: std_logic;
signal	data			: std_logic_vector(7 DOWNTO 0);
signal	addr			: std_logic_vector(15 DOWNTO 0);
signal	memwrite		: std_logic;
signal	memread		: std_logic;
signal	q				: std_logic_vector(7 DOWNTO 0);
signal	en				: std_logic;

constant PERIOD : time := 50 ns;

begin

	mem: memunit
	port map (
		clock 		=> clk, 
		datain 		=> data,
		memaddress 	=> addr, 
		memwrite 	=> memwrite,
		memread		=> memread,
		dataout 		=> q,
		enable		=> en);

	PROCESS
	BEGIN
		clk <= '0';
		wait for PERIOD/2;
		clk <= '1';
		wait for PERIOD/2;
	END PROCESS;
	
	PROCESS
   BEGIN
		en	<= '1';
		
		data <= X"0A";
		addr <= X"0000";
		memwrite <= '1';
		memread <= '0';
		wait for PERIOD/4;
		
		data <= X"1B";
		addr <= X"0001";
		wait for PERIOD;

		data <= X"2C";
		addr <= X"0002";
		wait for PERIOD;

		data <= X"3D";
		addr <= X"0003";
		wait for PERIOD;

		memwrite <= '0';
		memread <= '1';
		addr <= X"0000";
		wait for PERIOD;
		
		addr <= X"0001";
		wait for PERIOD;

		addr <= X"0002";
		wait for PERIOD;

		addr <= X"0003";
		wait for PERIOD;

		addr <= X"0004";
		wait for PERIOD;
		
		addr <= X"0002";
		wait for PERIOD;

		memread <= '0';
		wait for PERIOD;

-- set simulation time to 600ns		

		memread <= '1';

		addr <= X"0001";
		wait for PERIOD/2;
		
		
		addr <= X"0002";
		wait for PERIOD/2;
		

-- set simulation time to 650ns		

		en <= '0';
		wait for PERIOD *1.15;
		
		en <= '1';
		wait for PERIOD;

   END PROCESS;

-- set simulation time to 800ns		

end Behavioral;

