----------------------------------------------------------------------------------
-- Company: 		comp. eng. KKU.
-- Engineer: 		TK
-- 
-- Create Date:    14:46:23 08/31/2011 
-- Module Name:    regfile - Behavioral 
-- Description: 	 register file
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;

entity regfile is

port(
	readreg1:		in std_logic_vector (4 downto 0); 	-- index referring to a specific register
	readreg2:		in std_logic_vector (4 downto 0); 	-- index referring to a specific register
	writereg:		in std_logic_vector (4 downto 0); 	-- index referring to a specific register
	writedata:		in std_logic_vector (31 downto 0);	-- data to put in the specified register's content
	readdata1:		out std_logic_vector (31 downto 0);	-- data contained in register indexed by readreg1
	readdata2:		out std_logic_vector (31 downto 0);	-- data contained in register indexed by readreg2
	RegWrite:		in std_logic;
	clock:			in std_logic
);
end regfile;

architecture Behavioral of regfile is
	TYPE registers IS ARRAY(0 TO 2 ** 5 - 1) OF std_logic_vector(31 DOWNTO 0);

	SIGNAL reg_block : registers := (
    0  => x"00000000",
    1  => x"FFFFFFFF",
    2  => x"FFFFFFFF",
    3  => x"FFFFFFFF",
    4  => x"FFFFFFFF",
    5  => x"FFFFFFFF",
    6  => x"FFFFFFFF",
    7  => x"FFFFFFFF",
    8  => x"FFFFFFFF",
    9  => x"FFFFFFFF",
    10 => x"FFFFFFFF",
    11 => x"FFFFFFFF",
    12 => x"FFFFFFFF",
    13 => x"FFFFFFFF",
    14 => x"FFFFFFFF",
    15 => x"FFFFFFFF",
    16 => x"10000000",	-- base address for data segment
    17 => x"FFFFFFFF",
    18 => x"FFFFFFFF",
    19 => x"FFFFFFFF",
    20 => x"FFFFFFFF",
    21 => x"FFFFFFFF",
    22 => x"FFFFFFFF",
    23 => x"FFFFFFFF",
    24 => x"FFFFFFFF",
    25 => x"FFFFFFFF",
    26 => x"FFFFFFFF",
    27 => x"FFFFFFFF",
    28 => x"FFFFFFFF",
    29 => x"FFFFFFFF",
    30 => x"FFFFFFFF",
    31 => x"FFFFFFFF");
	
BEGIN
	PROCESS (clock, readreg1, readreg2)
	BEGIN

			readdata1 <= reg_block(to_integer(unsigned(readreg1)));			
			readdata2 <= reg_block(to_integer(unsigned(readreg2)));
	
			IF (RegWrite = '1') THEN
				reg_block(to_integer(unsigned(writereg))) <= writedata;
			END IF;
	
	END PROCESS;
END Behavioral;

