----------------------------------------------------------------------------------
-- Company: 			comp. engr. KKU.
-- Engineer: 			TK
-- 
-- Create Date:    	08:27:46 09/01/2011 
-- Design Name: 
-- Module Name:    	regfile_tb - Behavioral 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
-- "Memory can change the shape of a room; it can change the color of a car. 
-- And memories can be distorted. They're just an interpretation, they're not a record, 
-- and they're irrelevant if you have the facts." - Leonard Shelby in Memento (2000)
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.ALL;

entity regfile_tb is
end regfile_tb;

architecture Behavioral of regfile_tb is

component regfile is
port(
	readreg1:		in std_logic_vector (4 downto 0); 	-- index referring to a specific register
	readreg2:		in std_logic_vector (4 downto 0); 	-- index referring to a specific register
	writereg:		in std_logic_vector (4 downto 0); 	-- index referring to a specific register
	writedata:		in std_logic_vector (31 downto 0);	-- data to put in the specified register's content
	readdata1:		out std_logic_vector (31 downto 0);	-- data contained in register indexed by readreg1
	readdata2:		out std_logic_vector (31 downto 0);	-- data contained in register indexed by readreg2
	RegWrite:		in std_logic;
	clock:			in std_logic
);
end component;

signal readreg1	: std_logic_vector (4 downto 0); 	-- index referring to a specific register
signal readreg2	: std_logic_vector (4 downto 0); 	-- index referring to a specific register
signal writereg	: std_logic_vector (4 downto 0); 	-- index referring to a specific register
signal writedata	: std_logic_vector (31 downto 0);	-- data to put in the specified register's content
signal readdata1	: std_logic_vector (31 downto 0);	-- data contained in register indexed by readreg1
signal readdata2	: std_logic_vector (31 downto 0);	-- data contained in register indexed by readreg2
signal RegWrite	: std_logic;
signal clk			: std_logic;

constant PERIOD 		: time := 100 ns;
constant clkperiod 	: time := 30 ns;

begin

	regs: regfile
	port map (
		readreg1		=> readreg1,
		readreg2		=> readreg2,
		writereg		=> writereg,
		writedata	=> writedata,
		readdata1 	=> readdata1,
		readdata2	=> readdata2,
		RegWrite 	=> RegWrite,
		clock			=> clk);

	process
	begin
		clk <= '0';
		wait for clkperiod;
		clk <= '1';
		wait for clkperiod;
	end process;

	process
	variable regidx: integer :=0;
	
	begin

		writedata 	<= X"01FFAB00";
		writereg		<= "00001";
		wait for PERIOD/2;
		
		readreg1 <= "00000";
		readreg2 <= "00001";
		wait for PERIOD/2;

		RegWrite 	<= '1';
		writedata 	<= X"01FFAB00";
		writereg		<= "00001";
		wait for PERIOD/2;
		
		readreg1 <= "00000";
		readreg2 <= "00001";
		wait for PERIOD/2;

		readreg1 <= "10000";
		readreg2 <= "01000";
		wait for PERIOD/2;
		
		checkreg0: for regidx in 1 to 5 loop
			
			writereg <= std_logic_vector(to_unsigned(regidx, writereg'length));
			writedata <= std_logic_vector(to_unsigned(regidx*16, writedata'length));
			wait for PERIOD/2;
			
			readreg1 <= std_logic_vector(to_unsigned(regidx - 1, writereg'length));
			readreg2 <= std_logic_vector(to_unsigned(regidx, writereg'length));
			wait for PERIOD/2;
						
		end loop checkreg0;
		
		-- simulation time = 800 ns
		
	end process;


end Behavioral;

